#ifndef TRXVOL_H
#define TRXVOL_H
/*
   
   Volume and surface definitions

   Copyright  1992-2002 M.Kraemer, GSI Darmstadt 

   $Log: trxvol.h,v $
   Revision 1.3  2007/08/30 14:35:11  kraemer
   minor bug fixes / changes

   Revision 1.2  2002/11/25 16:38:48  kraemer
   surface/vol; volume mode; 2file

   Revision 1.1  2001/01/09 14:54:43  kraemer
   *** empty log message ***


*/

#include "m.h"
#include "cmderr.h"
#include "trxproj.h"
#include "trxxs.h"
#include "trxemfld.h"
                                     /* Common status and action flags */
                                      /* masks */
#define TRXVOL_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXVOL_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXVOL_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXVOL_DEBUG      0x80000000  /* general check flag */
#define TRXVOL_CHECK      TRXVOL_DEBUG/* general check flag */
#define TRXVOL_QUIET      0x10000000  /* suppress error messages */
#define TRXVOL_NOMSG      TRXVOL_QUIET

                                      /* Common macros */
#define TRXVOLCOSD( phi ) \
   (\
   ( (phi) == 0 ) ? 1E0 :\
   ( (phi) == 90 ) ? 0E0 :\
   ( (phi) == -90 ) ? 0E0 :\
   ( (phi) == 180 ) ? -1E0 :\
   ( (phi) == 180 ) ? -1E0 :\
   ( (phi) == 270 ) ? 0E0 :\
   ( (phi) == -270 ) ? 0E0 :\
   ( (phi) == 360 ) ? 1E0 :\
   ( (phi) == -360 ) ? 1E0 :\
   cos( (phi) * PI / 180E0 )\
   )

#define TRXVOLSIND( phi ) \
   (\
   ( (phi) == 0 ) ? 0E0 :\
   ( (phi) == 90 ) ? 1E0 :\
   ( (phi) == -90 ) ? -1E0 :\
   ( (phi) == 180 ) ?  0E0 :\
   ( (phi) == 180 ) ?  0E0 :\
   ( (phi) == 270 ) ? -1E0 :\
   ( (phi) == -270 ) ? 1E0 :\
   ( (phi) == 360 ) ? 0E0 :\
   ( (phi) == -360 ) ? 0E0 :\
   sin( (phi) * PI / 180E0 )\
   )

/*
   --- surface definitions
*/

                                      /* here's a surface */
struct STRXSRF {
   unsigned long ulFlag;              /* flag bits */
   unsigned long ulShape;             /* shape */
   unsigned long ulTopo;              /* upper/lower  */
   unsigned long ulSN;                /* neighbouring surface ID (1..n) */
   struct STRXVOL *psVN;              /* neighbouring volume */
   double dvNorm[3];                  /* normal vector */
};
                                      /* anti-surface */
#define TRXSRF_ANTI 0x00000001

                                      /* topology */
enum {
   TRXSRF_LOWER=0, TRXSRF_UPPER,
};                                          
                                      /* surface shapes */
enum {
   TRXSRF_CYL=1, TRXSRF_CIRCLE, TRXSRF_SPHERE, TRXSRF_RECT, TRXSRF_TORUS,
};

/*
   --- 
*/
                                      /* scoring */
struct STRXVOLSCORE {
   unsigned long ulvCount[MTRXPROJ][MTRXXS];    /* count # of interactions */  
   double        dvELoss [MTRXPROJ][MTRXXS];    /* count energy losses */ 
   double dEDeposit;                            /* deposited energy, eventwise */ 
   int nEvt;
   unsigned long *pulFano;                      /* [nEvt] # of ionizations for Fano factor */ 
};                                    
                                       
/*
   --- volume definitions
*/
                                      /* data offsets */
enum {
   TRXVOL_HEIGHT=0, 
   TRXVOL_XMIN, TRXVOL_XMAX, 
   TRXVOL_YMIN, TRXVOL_YMAX, 
   TRXVOL_ZMIN, TRXVOL_ZMAX, 
   TRXVOL_RMIN, TRXVOL_RMAX, 
   TRXVOL_R2MIN, TRXVOL_R2MAX, 
   TRXVOL_THMIN, TRXVOL_THMAX,
   TRXVOL_PHMIN, TRXVOL_PHMAX,
   TRXVOL_DX, TRXVOL_DY, TRXVOL_DZ,
   TRXVOL_AXTH, TRXVOL_AXPH, 
   TRXVOL_THICKNESS,
   MTRXVOLDATA 
};

                                      /* here's a volume */
struct STRXVOL {
   unsigned long ulFlag;              /* flag bits */
   unsigned long ulID;                /* vol number, for reference */
   unsigned long ulShape;             /* shape */
   unsigned long ulSpace;             /* occupied space (bytes) */
   double dVol;                       /* volume [cm**3] */
   double dSrf;                       /* surface [cm**2] */  
   double dMinDim;                    /* smallest dimension [cm] */
   double dDensity;                   /* local density (deviating from material) */ 
   double dvCutoff[MTRXPROJ];         /* low energy cutoff limit [keV or MeV/u] */  
   double dvData[MTRXVOLDATA];        /* volume limits data */
   double dvCentre[3];
   double dvAxis[3];
   double dvTrafo[4][4];
   void *pvUser;                      /* reserved for user info */
   char *pcName;                      /* volume  name */
   char *pcMaterial;                  /* material name */

#define MTRXVOLSRF 6
   int nS, mS;
   struct STRXSRF sS[MTRXVOLSRF];

   struct STRXVOLSCORE sScore;
   struct STRXVOLSCORE sScorePrim;    /* for primary particles */
   struct STRXVOLSCORE sScoreSec;     /* for secondary particles */
   struct STRXEMFLD sE;               /* electric field spec.s */
   struct STRXEMFLD sB;               /* magnetic field spec.s */
   struct SCMDERR sCE;
};
                                      /* flag bits */
#define TRXVOL_RECORD        0x00000001  /* */
#define TRXVOL_REFCOPY       0x00000002  /* don't free() */
//#define TRXVOL_CUTOFFKEEP    0x00000004  /* keep particles subject to cutoff */
//#define TRXVOL_CUTOFFELASTIC 0x00000008  /* */
 
                                      /* volume shapes */
enum {
   TRXVOL_CYL=1, TRXVOL_SPHERE, TRXVOL_BOX, TRXVOL_TORUS,
};


/*

   General:
   --------

   o where possible:
     surfaces w/ normal vectors, else n=0
     pointing outside volume,
     when intersection from outside: 
     dot product > 0: no intersection possible  


   Functions for single volumes: 
   -----------------------------
                            
   TRXVolComplete   : ( vol, flag ) 
                      o derive quantities to complete vol description
                    5 x to test
                      o to complete
                        o shapes
                          o torus
                          o surface description for box ?
                        o orientation
                        o angular limits
                      o calls
  
   TRXVolCopy         ( volfr, volto, flag )
                    4 x to test
                      o reference copy only
                      o calls
                        o TRXVolInit()
 
   TRXVolFree         ( vol, flag ) 
                      o release resources 
                    3 x to test
                      o calls  
                        o TRXVolInit()
                        o TRXVolScoreFanoFree()
                        o TRXEFldFree()
                        o TRXBFldFree()

   TRXVolInit         ( vol, flag ) 
                      o initialize 
                    2 x to test
                      o calls
                        o TRXEFldInit()
                        o TRXBFldInit()
  
   TRXVolInspect      ( vol, flag ) 
                      o inspect output
                    1 x to test
                      o calls
                        o TRXEFldInspect()
                        o TRXBFldInspect()
  
   TRXVolIsValid    : ( vol, flag ) 
                      o check if volume is minimum valid 
                    6 x to test
                      o to complete:
                        o shapes
                          o torus
                        o orientation
                        o angular limits
                      o calls
  
   TRXVolSpace      : ( vol, flag ) 
                      o count occupied space 
                   10 o to test
                      o calls
                        o TRXEMFldSpace() 
  
   TRXVolVolMatch   : ( vol, vol, flag ) 
                      o check volume/surface natch
                    9 o to complete
                      o sidewards checks
                      o angular checks
                      o sphere
                      o box
                      o torus
                      o calls
  
   TRXVol2File      : ( vol, fp, flag) 
                      o output volume data to file, native
                    7 x to test
                      o to complete:
                        o shapes
                          o torus
                        o orientation
                        o angular limits
                      o calls
                        o TRXVolIsValid()
                        o TRXEFld2File()
                        o TRXEFld2File()
                                                   
   TRXVol2Filegd    : ( vol, fp, flag )
                      o output volume data to file, gd format
                    8 o to do
                      o to complete:
                        o shapes
                          o torus
                        o orientation
                        o angular limits
                      o calls 
                        o TRXVolIsValid()


   Functions for volume scoring: 
   -----------------------------

   TRXVolScoreEvaluate : ( vol, nevt, flag )
                       4 x to test
                         o evaluate/list score counters
			 ! density per material
                         o calls 

   TRXVolScoreFanoAlloc: ( vol, nevt, flag  ) 
                       3 x to test
                         o allocate Fano recording array
                         x recording vol only
                         o calls  

   TRXVolScoreFanoFree : ( vol, flag ) 
                       2 x to test
                         o release Fano resources  
                         o calls  

   TRXVolScoreReset    : ( vol, flag )
                       1 x to test
                         o calls 


   Other functions
   ---------------

   TRXVolError      : error handling
 
*/
                                      /* function prototypes */
int TRXVolComplete( struct STRXVOL *, unsigned long );
int TRXVolCopy    ( struct STRXVOL *, struct STRXVOL *, unsigned long );
int TRXVolFree    ( struct STRXVOL *, unsigned long );
int TRXVolInit    ( struct STRXVOL *, unsigned long );
int TRXVolInspect ( struct STRXVOL *, unsigned long );
int TRXVolIsValid ( struct STRXVOL *, unsigned long );
int TRXVolSpace   ( struct STRXVOL *, unsigned long );
int TRXVolVolMatch( struct STRXVOL *, struct STRXVOL *, unsigned long );
int TRXVol2Filegd ( struct STRXVOL *, FILE *, unsigned long );
int TRXVol2File   ( struct STRXVOL *, FILE *, unsigned long );

struct STRXVOL *_TRXVolTest( unsigned long );
struct STRXVOL *_TRXVolTestCyl1( unsigned long );
struct STRXVOL *_TRXVolTestCyl2( unsigned long );
struct STRXVOL *_TRXVolTestBox1( unsigned long );
struct STRXVOL *_TRXVolTestBox2( unsigned long );
struct STRXVOL *_TRXVolTestSphere1( unsigned long );
struct STRXVOL *_TRXVolTestSphere2( unsigned long );

int TRXVolScoreEvaluate ( struct STRXVOL *, unsigned long, unsigned long );
int TRXVolScoreFanoAlloc( struct STRXVOL *, unsigned long, unsigned long );
int TRXVolScoreFanoFree ( struct STRXVOL *, unsigned long );
int TRXVolScoreReset    ( struct STRXVOL *, unsigned long );



#define TRXVOLEVT_FILEVERSION 20020830
                                      /* vol event data */
struct STRXVOLEVTDATA {
   unsigned long ulVol;               /* Vol ID */
   float           rdE;               /* energy loss [keV] */
   float           rt;                /* time */
};
                                      /* vol event header */
struct STRXVOLEVTHEAD {
   unsigned long ulVersion, ulLength; /* version, length of header+data */
   unsigned short uiByteSex;          /* =1 for native endian, else byte swap */
   unsigned short uiMode;             /* always=1 for volmode evt */
   unsigned long ulEvt;               /* current/max primary event number */
            long  lOR;                /* BOR/EOR */
   int nD;                            /* current/max # of data */
};
                                      /* vol event output */
struct STRXVOLEVT {
   struct STRXVOLEVTHEAD   sH;	    
   struct STRXVOLEVTDATA *psD;	    
};
#define TRXVOLEVTFREE(psVE)\
   if ( (psVE)->psD != NULL ) free((psVE)->psD); memset(psVE,0,sizeof(*(psVE))); 
/*

   Volume event data functions
   ---------------------------
   
   TRXVolEvtInit     ( ve, flag )
                     o initialize
                     o to do
                     o calls
   
   TRXVolEvtInspect  ( ve, flag )
                     o inspect
                     o to do
                     o calls
   

*/
int TRXVolEvtInit   ( struct STRXVOLEVT *, unsigned long );
int TRXVolEvtInspect( struct STRXVOLEVT *, unsigned long );

struct STRXVOLEVT *_TRXVolEvtTest( unsigned long );
 
 

                                      /* error handling */
int TRXVolError( struct SCMDERR *, unsigned long );

enum { ERRTRXVOL_SYS=1,
       ERRTRXVOL_BADADDR,
       ERRTRXVOL_BADNEVT,
       ERRTRXVOL_NOSUCHSHAPE,
       ERRTRXVOL_BADVOLTYPE,
       ERRTRXVOL_BADVOLNAME,
       ERRTRXVOL_BADVOLMATERIAL,
       ERRTRXVOL_BADHEIGHT,
       ERRTRXVOL_BADRADIUS,
       ERRTRXVOL_BADANGLE,
       ERRTRXVOL_BADSIZE,
       ERRTRXVOL_BADTHICKNESS,
};

#endif
